using System;
using System.Collections;
using System.Text;
using System.Data;
using System.Data.OleDb;

namespace Telecomunicacao
{
    public interface ICartao
    {
		/// <summary>
		/// criao de um novo carto
		/// </summary>
		/// <param name="user"> nome do utilizador</param>
		/// <param name="pwd"> password do utilizador</param>
		/// <param name="nrtelefone"> nmero de telefone associado ao carto</param>
		/// <param name="idtarifario"> tarifrio a usar pelo novo carto</param>
		/// <param name="tipo"> tipo de carto</param>
		/// <returns> status da operao</returns>
		/// <pre> utilizador valido, idtarifario valido</pre>
		/// <post> </post>
		/// <returns>status da operao</returns>
		StatusEnum criar(string user, string pwd, string nrtelefone, string idtarifario, TipoCartao tipo);

		/// <summary>
		/// Activao de um carto
		/// </summary>
		/// <param name="user">nome do utilizador</param>
		/// <param name="pwd"> password do utilizador </param>
		/// <param name="nrtelefone">nmero de telefone associado ao carto</param>
		/// <param name="nrcontrib"> nmero de contribuinte do cliente</param>
		/// <pre> utilizador vlido, nmero de contribuinte vlido, nmero de telefone vlido</pre>
		/// <post> o carto passa a estar no estado=ACTIVO</post>
		/// <returns> status da operao </returns>
		StatusEnum activar(string user, string pwd, string nrtelefone, string nrcontrib);

		/// <summary>
		/// Activao de um carto recarregavel
		/// </summary>
		/// <param name="user">nome do utilizador</param>
		/// <param name="pwd"> password do utilizador </param>
		/// <param name="nrtelefone">nmero de telefone associado ao carto</param>
		/// <pre> utilizador vlido, nmero de telefone vlido, tipodecarto=RECARREGAVEL</pre>
		/// <post> o carto passa a estar no estado=ACTIVO</post>
		/// <returns> status da operao </returns>
		StatusEnum activar(string user, string pwd, string nrtelefone);

        /// <summary>
        /// Alterao dos dados de um carto
        /// </summary>
        /// <param name="user">nome do utilizador</param>
        /// <param name="pwd"> password do utilizador</param>
        /// <param name="nrtelefone">nmero de telefone associado ao carto</param>
        /// <param name="idtarifario">tarifrio a usar pelo carto</param>
        /// <pre> utilizador valido, numero de telefone valido, idtarifario valido</pre>
        /// <post> os dados do carto ficarem alterados </post>
        /// <returns>status da operao</returns>
        StatusEnum alterar(string user, string pwd, string nrtelefone, string idtarifario);

        /// <summary>
        /// Remoo de um carto
        /// </summary>
        /// <param name="user"> nome do utilizador</param>
        /// <param name="pwd"> password do utilizador</param>
        /// <param name="nrtelefone">nmero de telefone do carto a remover</param>
        /// <pre> utilizador valido, numero de telefone valido, carto no pode ter debitos</pre>
        /// <post> a remoo do carto com aquele numero de telefone </post>
        /// <returns>status da operao</returns>
        StatusEnum remover(string user, string pwd, string nrtelefone);

        /// <summary>
        /// Cancelamento de um carto
        /// </summary>
        /// <param name="user">nome do utilizador</param>
        /// <param name="pwd">password do utilizador</param>
        /// <param name="nrtelefone">nmero do telefone a cancelar</param>
        /// <pre> utilizador valido, numero de telefone valido </pre>
        /// <post> o carto ficar cancelado</post>
        /// <returns>status da operao</returns>
        StatusEnum cancelar(string user, string pwd, string nrtelefone);

        /// <summary>
        /// Facturar um nmero de telefone
        /// </summary>
        /// <param name="user">nome do utilizador</param>
        /// <param name="pwd">password do utilizador</param>
        /// <param name="nrtelefone">nmero de telefone a facturar</param>
        /// <pre> utilizador valido, numero de telefone valido, carto tem que ser de contracto </pre>
        /// <post> nao alterar os dados </post>
        /// <returns>retorna o nome do documento criado</returns>
        string factura(string user, string pwd, string nrtelefone);

        /// <summary>
        /// obter a listagem de chamadas efectuadas
        /// </summary>
        /// <param name="user"> nome do utilizador </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="nrtelefone"> numero de telefone que se pretende a chamada </param>
        /// <pre> utilizador valido, numero de telefone valido </pre>
        /// <post> nao alterar os dados </post>
        /// <returns> retorna o nome do documento criado </returns>
        string extracto(string user, string pwd, string nrtelefone);
    }

	internal class CartaoImpl : ICartao
	{
		private string OUTDIR_DOC = System.IO.Directory.GetCurrentDirectory();
		#region ICartao

		/// <summary>
		/// criao de um novo carto
		/// </summary>
		/// <param name="user"> nome do utilizador</param>
		/// <param name="pwd"> password do utilizador</param>
		/// <param name="nrtelefone"> nmero de telefone associado ao carto</param>
		/// <param name="idtarifario"> tarifrio a usar pelo novo carto</param>
		/// <param name="tipo"> tipo de carto</param>
		/// <returns> status da operao</returns>
		/// <pre> utilizador valido, idtarifario valido</pre>
		/// <post> </post>
		/// <returns>status da operao</returns>
		public StatusEnum criar(string user, string pwd, string nrtelefone, string idtarifario, TipoCartao tipo)
		{
			long id1;
			StatusEnum status;
			OleDbConnection conn = null;
			TipoUtilizador tipoutil;

			//
			// validar dados de entrada
			//
			if (nrtelefone == null || nrtelefone.Trim().Length == 0)
			{
				status = StatusEnum.INVALID_KEY;
				return status;
			}
			if(idtarifario == null)
			{
				return StatusEnum.INVALID_ARGUMENT;
			}

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//verificar se o Cartao j existe
				DataSet ds = UtilDB.GetById(conn, null, "Cartao", "nrTelefone", nrtelefone);
				if (ds.Tables["Cartao"].Rows.Count != 0)
					return StatusEnum.INVALID_KEY;

				try
				{
					id1 = long.Parse(idtarifario);
				}
				catch(Exception)
				{
					return StatusEnum.ERROR;
				}
				//verificar se o tarifario existe
				DataSet ds2 = UtilDB.GetById(conn, null, "Tarifario", "id", id1);
				if (ds2.Tables["Tarifario"].Rows.Count == 0)
					return StatusEnum.INVALID_ARGUMENT;

				//
				// criar comando SQL a executar
				//
				string sqlCmd = "INSERT INTO Cartao (nrTelefone, tarifario, recarregavel, estado) Values (?, ?, ?, ?)";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
				cmd.Parameters.Add("nrTelefone", nrtelefone);
				cmd.Parameters.Add("tarifario", idtarifario);
				cmd.Parameters.Add("recarregavel", tipo);
				cmd.Parameters.Add("estado",EstadosCartao.NO_ATRIBUIDO);
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = StatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					//
					status = StatusEnum.OK;		
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <criar> da classe 'Cartao': " + ex.Message);
				status = StatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		/// <summary>
		/// Activao de um carto
		/// </summary>
		/// <param name="user">nome do utilizador</param>
		/// <param name="pwd"> password do utilizador </param>
		/// <param name="nrtelefone">nmero de telefone associado ao carto</param>
		/// <param name="nrcontrib"> nmero de contribuinte do cliente</param>
		/// <pre> utilizador vlido, nmero de contribuinte vlido, nmero de telefone vlido</pre>
		/// <post> o carto passa a estar no estado=ACTIVO</post>
		/// <returns> status da operao </returns>
		public StatusEnum activar(string user, string pwd, string nrtelefone, string nrcontrib)
		{
			StatusEnum status;
			OleDbConnection conn = null;
			TipoUtilizador tipoutil;

			//
			// validar dados de entrada
			//
			if (nrtelefone == null || nrtelefone.Trim().Length == 0)
			{
				status = StatusEnum.INVALID_KEY;
				return status;
			}
			if (nrcontrib == null || nrcontrib.Trim().Length == 0)
			{
				status = StatusEnum.INVALID_CLIENTE_ID;
				return status;
			}

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//verificar se o Cartao existe
				DataSet ds = UtilDB.GetById(conn, null, "Cartao", "nrTelefone", nrtelefone);
				if (ds.Tables["Cartao"].Rows.Count == 0)
					return StatusEnum.INVALID_KEY;
				else
				{					
					Int16 t = (Int16)ds.Tables["Cartao"].Rows[0]["estado"];
					if((EstadosCartao)t!=EstadosCartao.NO_ATRIBUIDO && (EstadosCartao)t!=EstadosCartao.INACTIVO)
						return StatusEnum.ERRO_ESTADO_CARTAO;
				}

				//verificar se o Cliente existe
				DataSet ds1 = UtilDB.GetById(conn, null, "Cliente", "contribuinte", nrcontrib);
				if (ds1.Tables["Cliente"].Rows.Count == 0)
					return StatusEnum.INVALID_CLIENTE_ID;

				//
				// criar comando SQL a executar
				//
				string sqlCmd = "UPDATE Cartao SET nrContrib = ?, estado = ? WHERE nrTelefone = ?";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
				cmd.Parameters.Add("nrContrib", nrcontrib);
				cmd.Parameters.Add("estado", EstadosCartao.ACTIVO);
				cmd.Parameters.Add("nrTelefone",nrtelefone);
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = StatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					//
					status = StatusEnum.OK;		
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <activar> da classe 'Cartao': " + ex.Message);
				status = StatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		/// <summary>
		/// Activao de um carto recarregavel
		/// </summary>
		/// <param name="user">nome do utilizador</param>
		/// <param name="pwd"> password do utilizador </param>
		/// <param name="nrtelefone">nmero de telefone associado ao carto</param>
		/// <pre> utilizador vlido, nmero de telefone vlido, tipodecarto=RECARREGAVEL</pre>
		/// <post> o carto passa a estar no estado=ACTIVO</post>
		/// <returns> status da operao </returns>
		public StatusEnum activar(string user, string pwd, string nrtelefone)
		{
			StatusEnum status;
			OleDbConnection conn = null;
			TipoUtilizador tipoutil;

			//
			// validar dados de entrada
			//
			if (nrtelefone == null || nrtelefone.Trim().Length == 0)
			{
				status = StatusEnum.INVALID_KEY;
				return status;
			}

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//verificar se o Cartao existe
				DataSet ds = UtilDB.GetById(conn, null, "Cartao", "nrTelefone", nrtelefone);
				if (ds.Tables["Cartao"].Rows.Count == 0)
					return StatusEnum.INVALID_KEY;
				else
				{					
					Int16 t = (Int16)ds.Tables["Cartao"].Rows[0]["estado"];
					if((EstadosCartao)t!=EstadosCartao.NO_ATRIBUIDO && (EstadosCartao)t!=EstadosCartao.INACTIVO)
						return StatusEnum.ERRO_ESTADO_CARTAO;
					TipoCartao rec = (TipoCartao)ds.Tables["Cartao"].Rows[0]["recarregavel"];
					if(rec!=TipoCartao.RECARREGAVEL)
						return StatusEnum.TIPO_CARTO_INVLIDO;
				}

				//
				// criar comando SQL a executar
				//
				string sqlCmd = "UPDATE Cartao SET estado = ? WHERE nrTelefone = ?";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
				cmd.Parameters.Add("estado", EstadosCartao.ACTIVO);
				cmd.Parameters.Add("nrTelefone",nrtelefone);
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = StatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					//
					status = StatusEnum.OK;		
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <activar> da classe 'Cartao': " + ex.Message);
				status = StatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		/// <summary>
		/// Alterao dos dados de um carto
		/// </summary>
		/// <param name="user">nome do utilizador</param>
		/// <param name="pwd"> password do utilizador</param>
		/// <param name="nrtelefone">nmero de telefone associado ao carto</param>
		/// <param name="idtarifario">tarifrio a usar pelo carto</param>
		/// <pre> utilizador valido, numero de telefone valido, idtarifario valido</pre>
		/// <post> os dados do carto ficarem alterados </post>
		/// <returns>status da operao</returns>
		public StatusEnum alterar(string user, string pwd, string nrtelefone, string idtarifario)
		{
			StatusEnum status;
			OleDbConnection conn = null;
			TipoUtilizador tipoutil;
			long id1;

			//
			// validar dados de entrada
			//
			if (nrtelefone == null || nrtelefone.Trim().Length == 0)
			{
				status = StatusEnum.INVALID_KEY;
				return status;
			}
			if(idtarifario == null)
			{
				return StatusEnum.INVALID_ARGUMENT;
			}

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//verificar se o Cartao existe
				DataSet ds = UtilDB.GetById(conn, null, "Cartao", "nrTelefone", nrtelefone);
				if (ds.Tables["Cartao"].Rows.Count == 0)
					return StatusEnum.INVALID_KEY;

				//verificar se o tarifario existe
				try
				{
					id1 = long.Parse(idtarifario);
				}
				catch(Exception)
				{
					return StatusEnum.ERROR;
				}
				DataSet ds2 = UtilDB.GetById(conn, null, "Tarifario", "id", id1);
				if (ds2.Tables["Tarifario"].Rows.Count == 0)
					return StatusEnum.INVALID_ARGUMENT;

				//
				// criar comando SQL a executar
				//
				string sqlCmd = "UPDATE Cartao SET tarifario = ? WHERE nrTelefone = ?";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
				cmd.Parameters.Add("tarifario", idtarifario);
				cmd.Parameters.Add("nrTelefone",nrtelefone);
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = StatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					//
					status = StatusEnum.OK;		
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <alterar> da classe 'Cartao': " + ex.Message);
				status = StatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		/// <summary>
		/// Remoo de um carto
		/// </summary>
		/// <param name="user"> nome do utilizador</param>
		/// <param name="pwd"> password do utilizador</param>
		/// <param name="nrtelefone">nmero de telefone do carto a remover</param>
		/// <pre> utilizador valido, numero de telefone valido, carto no pode ter debitos</pre>
		/// <post> o carto fica no estado=REMOVIDO </post>
		/// <returns>status da operao</returns>
		public StatusEnum remover(string user, string pwd, string nrtelefone)
		{
			StatusEnum status;
			OleDbConnection conn = null;
			TipoUtilizador tipoutil;
			double quantia;

			//
			// validar dados de entrada
			//
			if (nrtelefone == null || nrtelefone.Trim().Length == 0)
			{
				status = StatusEnum.INVALID_KEY;
				return status;
			}

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//verificar se o Cartao existe
				DataSet ds = UtilDB.GetById(conn, null, "Cartao", "nrTelefone", nrtelefone);
				if (ds.Tables["Cartao"].Rows.Count == 0)
					return StatusEnum.INVALID_KEY;
				else
				{					
					Int16 t = (Int16)ds.Tables["Cartao"].Rows[0]["estado"];
					if((EstadosCartao)t!=EstadosCartao.ACTIVO)
						return StatusEnum.ERRO_ESTADO_CARTAO;
					TipoCartao rec = (TipoCartao)ds.Tables["Cartao"].Rows[0]["recarregavel"];
					if(rec==TipoCartao.MENSAL)
						if(this.debitos(nrtelefone,out quantia))
							return StatusEnum.TEM_DEBITOS;
				}

				//
				// criar comando SQL a executar
				//
				string sqlCmd = "UPDATE Cartao SET estado = ? WHERE nrTelefone = ?";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
				cmd.Parameters.Add("estado", EstadosCartao.REMOVIDO);
				cmd.Parameters.Add("nrTelefone",nrtelefone);
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = StatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					//
					status = StatusEnum.OK;		
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <remover> da classe 'Cartao': " + ex.Message);
				status = StatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		/// <summary>
		/// Cancelamento de um carto
		/// </summary>
		/// <param name="user">nome do utilizador</param>
		/// <param name="pwd">password do utilizador</param>
		/// <param name="nrtelefone">nmero do telefone a cancelar</param>
		/// <pre> utilizador valido, numero de telefone valido </pre>
		/// <post> o carto fica no estado=INACTIVO</post>
		/// <returns>status da operao</returns>
		public StatusEnum cancelar(string user, string pwd, string nrtelefone)
		{
			StatusEnum status;
			OleDbConnection conn = null;
			TipoUtilizador tipoutil;

			//
			// validar dados de entrada
			//
			if (nrtelefone == null || nrtelefone.Trim().Length == 0)
			{
				status = StatusEnum.INVALID_KEY;
				return status;
			}

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//verificar se o Cartao existe
				DataSet ds = UtilDB.GetById(conn, null, "Cartao", "nrTelefone", nrtelefone);
				if (ds.Tables["Cartao"].Rows.Count == 0)
					return StatusEnum.INVALID_KEY;
				else
				{					
					Int16 t = (Int16)ds.Tables["Cartao"].Rows[0]["estado"];
					if((EstadosCartao)t!=EstadosCartao.ACTIVO)
						return StatusEnum.ERRO_ESTADO_CARTAO;
				}

				//
				// criar comando SQL a executar
				//
				string sqlCmd = "UPDATE Cartao SET estado = ? WHERE nrTelefone = ?";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
				cmd.Parameters.Add("estado", EstadosCartao.INACTIVO);
				cmd.Parameters.Add("nrTelefone",nrtelefone);
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = StatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					//
					status = StatusEnum.OK;		
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <cancelar> da classe 'Cartao': " + ex.Message);
				status = StatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		/// <summary>
		/// Facturar um nmero de telefone
		/// </summary>
		/// <param name="user">nome do utilizador</param>
		/// <param name="pwd">password do utilizador</param>
		/// <param name="nrtelefone">nmero de telefone a facturar</param>
		/// <pre> utilizador valido, numero de telefone valido, carto tem que ser de contracto </pre>
		/// <post> nao alterar os dados </post>
		/// <returns>retorna o nome do documento criado</returns>
		public string factura(string user, string pwd, string nrtelefone)
		{
			StatusEnum status;
			OleDbConnection conn = null;
			TipoUtilizador tipoutil;
			string doc="";

			//
			// validar dados de entrada
			//
			if (nrtelefone == null || nrtelefone.Trim().Length == 0)
			{
				return "";
			}
			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return "";

				//verificar se o Cartao existe
				DataSet ds = UtilDB.GetById(conn, null, "Cartao", "nrTelefone", nrtelefone);
				if (ds.Tables["Cartao"].Rows.Count == 0)
					return "";
				else
				{
					//verificar se  um carto de mensalidade
					if((TipoCartao)ds.Tables["Cartao"].Rows[0]["recarregavel"]!=TipoCartao.MENSAL)
						return "";
				}
				//criar documento word
				if(this.GenerateWordDocumentInvoice(user,pwd,nrtelefone,TiposDocumentos.FACTURA,out doc)!=StatusEnum.OK)
					doc="";
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Factura> da classe 'Cartao': " + ex.Message);
				doc="";
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}
			return doc;
		}

		/// <summary>
		/// obter a listagem de chamadas efectuadas
		/// </summary>
		/// <param name="user"> nome do utilizador </param>
		/// <param name="pwd"> password do utilizador </param>
		/// <param name="nrtelefone"> numero de telefone que se pretende a chamada </param>
		/// <pre> utilizador valido, numero de telefone valido </pre>
		/// <post> nao alterar os dados </post>
		/// <returns> retorna o nome do documento criado </returns>
		public string extracto(string user, string pwd, string nrtelefone)
		{
			StatusEnum status;
			OleDbConnection conn = null;
			TipoUtilizador tipoutil;
			string doc="";

			//
			// validar dados de entrada
			//
			if (nrtelefone == null || nrtelefone.Trim().Length == 0)
			{
				return "";
			}
			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return "";

				//verificar se o Cartao existe
				DataSet ds = UtilDB.GetById(conn, null, "Cartao", "nrTelefone", nrtelefone);
				if (ds.Tables["Cartao"].Rows.Count == 0)
					return "";
					/*
				else
				{
					//verificar se  um carto de mensalidade
					if((TipoCartao)ds.Tables["Cartao"].Rows[0]["recarregavel"]==TipoCartao.MENSAL)
						return "";
				}
				*/
				//criar documento word
				if(this.GenerateWordDocumentInvoice(user,pwd,nrtelefone,TiposDocumentos.EXTRACTO,out doc)!=StatusEnum.OK)
					doc="";
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Extracto> da classe 'Cartao': " + ex.Message);
				doc="";
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}
			return doc;
		}

		#endregion

		#region mtodos_internos

		public DataSet GetCartao(string nrtelefone)
		{
			OleDbConnection conn = null;
			DataSet ds = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				//
				// efectuar pesquisa
				//
				ds = UtilDB.GetById(conn, null, "Cartao", "nrTelefone", nrtelefone);
			}
			catch (OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetCartao> da classe 'Cartao': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;

		}

		internal bool debitos(string nrtelefone, out double quantia)
		{
			bool aux = true;
			quantia = 0;
			OleDbConnection conn = null;

			try
			{
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				DataSet ds = UtilDB.GetById(conn,null,"Cartao","nrTelefone", nrtelefone);

				foreach (DataRow regCartoes in ds.Tables["Cartao"].Rows)
				{
					if (!(bool)regCartoes["recarregavel"])
					{
						quantia += (double)regCartoes["saldo"];
					}
				}

				if (quantia == 0)
				{
					aux = false;
				}
				return aux;
			}
			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo 'Debitos': " + ex.Message);
				return false;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}
		}

		internal StatusEnum GenerateWordDocumentInvoice(string User, string Pass, string nrtelefone, TiposDocumentos tipo, out string filename)
		{
			StatusEnum status;
			object missingValue = Type.Missing;

			object uchar = Word.WdUnits.wdCharacter;
			object uline = Word.WdUnits.wdLine;
			object ucol = Word.WdUnits.wdColumn;
			object i1 = 1;
			object i4 = 4;
			object bfalse = false;
			object btrue = true;

			filename = null;

			string customerName = "";
			string customerContrib = "";
			string tarifario = "";
			double totalFactura = 0;

			// obter factura
			DataSet dsFactura = GetById(User, Pass, nrtelefone, tipo);
			if (dsFactura == null)
				return StatusEnum.ERROR;

			//adquirir um codigo para o documento
			int cod = this.GetCodDocumentos(tipo); 
			if(cod==-1)
				return StatusEnum.INVALID_KEY;
			string nFactura=cod.ToString();
			DateTime dataFactura = System.DateTime.Now;
			if(dsFactura.Tables["Cartao"].Rows.Count==0)
				return StatusEnum.ERROR;

			if(tipo==TiposDocumentos.FACTURA)
			{
				customerName = (string)dsFactura.Tables["Cartao"].Rows[0]["nome"];
				customerContrib = (string)dsFactura.Tables["Cartao"].Rows[0]["nrContrib"];
			}
			tarifario = (string)dsFactura.Tables["Cartao"].Rows[0]["Tarifario"];
			//totalFactura = (double)dsFactura.Tables["Cartao"].Rows[0]["SomaCusto"];

			// gerar nome de ficheiro
			object _in_filename;
			if(tipo==TiposDocumentos.FACTURA)
				_in_filename = OUTDIR_DOC+"\\..\\Factura"+ nFactura + ".doc";
			else
				_in_filename = OUTDIR_DOC+"\\..\\Extracto"+ nFactura + ".doc";

			// criar objecto Word
			Word.Application w = new Word.ApplicationClass();

			// criar novo documento
			w.Documents.Add(ref missingValue, ref missingValue, ref missingValue, ref missingValue);

			//w.Visible = true;	// s para teste/"debug"!!!!!

			if(tipo == TiposDocumentos.FACTURA)
				w.Selection.TypeText("Factura n ");
			else
				w.Selection.TypeText("Extracto n ");
			w.Selection.TypeText(nFactura);
			w.Selection.TypeParagraph();
			w.Selection.TypeText("Nm. Telefone : ");
			w.Selection.TypeText(nrtelefone);
			w.Selection.TypeParagraph();
			w.Selection.TypeText("Data : ");
			w.Selection.TypeText(dataFactura.ToLongDateString());
			w.Selection.TypeParagraph();
			if(tipo == TiposDocumentos.FACTURA)
			{
				w.Selection.TypeText("Contribuinte : ");
				w.Selection.TypeText(customerContrib);
				w.Selection.TypeParagraph();
				w.Selection.TypeText("Cliente : ");
				w.Selection.TypeText(customerName);
				w.Selection.TypeParagraph();
			}
			w.Selection.TypeParagraph();

			w.Selection.TypeText("Tarifrio : ");
			w.Selection.TypeText(tarifario);
			w.Selection.TypeParagraph();
			w.Selection.TypeParagraph();

			//
			// criar tabela (cabealho)
			//
			object tb = Word.WdDefaultTableBehavior.wdWord9TableBehavior;
			object afb = Word.WdAutoFitBehavior.wdAutoFitWindow;
			w.ActiveDocument.Tables.Add(w.Selection.Range, 1, 4, ref tb, ref afb);

			w.Selection.TypeText("Destino");
			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			w.Selection.TypeText("Inicio");
			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			w.Selection.TypeText("Fim");
			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			w.Selection.TypeText("Sub-Total");

			//
			// adicionar as linhas de factura
			//
			foreach (DataRow dr in dsFactura.Tables["CartaoDetalhes"].Rows)
			{
				// adicionar linha  tabela
				w.ActiveDocument.Tables.Item(1).Rows.Add(ref missingValue);
				w.Selection.MoveDown(ref uline, ref i1, ref missingValue);
				//w.Selection.MoveLeft(ref uchar, ref i4, ref missingValue);

				w.Selection.TypeText(dr["nrTelDestino"].ToString());
				w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
				w.Selection.TypeText(dr["inicio"].ToString());
				w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
				w.Selection.TypeText(dr["fim"].ToString());
				w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
				w.Selection.TypeText(dr["custo"].ToString());
				w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);

				totalFactura+=(double)dr["custo"];
			}
			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			//
			// formatar tabela
			//
			object tf = Word.WdTableFormat.wdTableFormatProfessional;
			w.ActiveDocument.Tables.Item(1).AutoFormat(ref tf, ref btrue, ref btrue, ref btrue, ref bfalse, ref btrue, ref bfalse, ref bfalse, ref bfalse, ref bfalse);

			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			w.Selection.MoveDown(ref uline, ref i1, ref missingValue);
			//w.Selection.TypeParagraph();
			//w.Selection.TypeParagraph();
			w.Selection.TypeText("Total :");
			w.Selection.TypeText(totalFactura.ToString());
			w.Selection.TypeParagraph();

			// rodap da factura
			w.Selection.TypeParagraph();
			w.Selection.TypeText("Gerada pelo trabalho de Telecomunicaes do Grupo 5 - 4BN");

			// gravar ficheiro
			//w.ActiveDocument.SaveAs(ref _in_filename, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue);
			w.ActiveDocument.SaveAs(ref _in_filename, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue);
			//terminar Word
			w.Quit(ref missingValue, ref missingValue, ref missingValue);

			filename = (string)_in_filename;

			status=this.AddDocumento(cod, tipo, customerContrib, nrtelefone);
            
			return status;
		}

		internal DataSet GetById(string User, string Pass, string nrtelefone, TiposDocumentos tipo)
		{
			DataSet ds = null;
			OleDbConnection conn = null;

			try
			{
				//
				// criar objecto DataSet
				//
				ds = new DataSet();

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// cabealhos das vendas
				//
				string sSqlCmd="";
				if(tipo == TiposDocumentos.FACTURA)
					sSqlCmd = "SELECT * FROM vwTotalCartao WHERE nrTelefone=?";
				else
					sSqlCmd = "SELECT * FROM vwTotalCartao2 WHERE nrTelefone=?";

				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                
				oAdapter.SelectCommand.Parameters.Add("nrTelefone", nrtelefone);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, "Cartao");

				int aux=ds.Tables["Cartao"].Rows.Count;
				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// linhas de detalhe
				//
				sSqlCmd = "SELECT * FROM vwDetalhesCartao WHERE nrTelefone=?";

				oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Parameters.Add("nrTelefone", nrtelefone);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, "CartaoDetalhes");

				int aux1=ds.Tables["CartaoDetalhes"].Rows.Count;
				//
				// criar relao entre tabelas
				//
				if(ds.Tables["Cartao"].Rows.Count>0 && ds.Tables["CartaoDetalhes"].Rows.Count>0)
					ds.Relations.Add("Details", ds.Tables["Cartao"].Columns["nrTelefone"], ds.Tables["CartaoDetalhes"].Columns["nrTelefone"]);
			}
			catch (OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo 'GetById': " + ex);
				ds = null;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		}

		internal StatusEnum AddDocumento(int codigo, TiposDocumentos tipo, string cliente, string nrTelefone)
		{
			OleDbConnection conn=null;

			if(codigo<=0)
				return StatusEnum.ERROR;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				//Construir string para insero
				//

				string sqlCmd = "INSERT INTO Documentos(codDocumento, tipo, cliente, nrtelefone) VALUES(?,?,?,?)";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
				cmd.Parameters.Add("codDocumento", codigo);
				cmd.Parameters.Add("tipo", tipo);
				if(cliente!="")
					cmd.Parameters.Add("cliente", cliente);
				else
					cmd.Parameters.Add("cliente", DBNull.Value);
				cmd.Parameters.Add("nrtelefone", nrTelefone);
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();
			}
			catch (OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo 'AddDocumento': " + ex.Message);
				return StatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}
			
			return StatusEnum.OK;
		}


		internal int GetCodDocumentos(TiposDocumentos tipo)
		{
			int cod=0;
			DataSet ds = null;
			OleDbConnection conn = null;

			try
			{
				//
				// criar objecto DataSet
				//
				ds = new DataSet();

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				int aux=(int)tipo;
				string sSqlCmd = "select max(codDocumento) from Documentos where tipo="+aux.ToString();
				OleDbCommand cmd = new OleDbCommand(sSqlCmd,conn);

				//
				// executar o comando
				//
				string cont=cmd.ExecuteScalar().ToString();

				if(cont!="")
				{
					cod=int.Parse(cont);
				}
				else
				{
					cod=0;
				}
				cod++;
			}
			catch (OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo 'GetCodDocumentos': " + ex.Message);
				cod=-1;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return cod;
		}
		#endregion

    }

}
