using System;
using System.Collections;
using System.Text;
using System.Data;
using System.Data.OleDb;

namespace Telecomunicacao
{
    public interface IChamada
    {
        /// <summary>
        /// Permite inserir uma chamada num determinado carto
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador</param>
        /// <param name="inicio"> data e hora do inicio da chamada</param>
        /// <param name="fim"> data e hora do final da chamada</param>
        /// <param name="nrtelefone"> numero do telefone que efectua a chamada valido </param>
        /// <param name="numdestino"> numero de destino da chamada </param>
        /// <pre> utilizador valido, numero de telefone valido, data inicio e data fim validas</pre>
		/// <post> chamada inserida; retorno de um identificador valido</post>
        /// <returns> retorna o estado da insero da chamada; retorna o id da chamada </returns>
        StatusEnum Chamada(string user, string pwd, DateTime inicio, DateTime fim, string nrtelefone, string numdestino,out long id);

        /// <summary>
        /// Requerer os dados das chamadas de um carto
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador</param>
        /// <param name="nrtelefone"> numero do telefone que se pretende o registo das chamadas, valido</param>
        /// <pre> utilizador valido, numero de telefone valido </pre>
        /// <post> nao alterao dos dados </post>
        /// <returns>retorna os dados das chamadas efectuadas por um determinado numero de telefone </returns>
        DataSet GetByCartao(string user, string pwd, string nrtelefone);

		/// <summary>
		/// Requerer os dados das chamadas de um cliente (1 ou mais cartoes)
		/// </summary>
		/// <param name="user"> nome do utilizador valido </param>
		/// <param name="pwd"> password do utilizador</param>
		/// <param name="nrcontribuinte"> numero de contribuinte que se pretende o registo das chamadas, valido</param>
		/// <post> nao alterao dos dados </post>
		/// <returns>retorna os dados das chamadas efectuadas por um determinado cliente </returns>
		DataSet GetByCliente(string user, string pwd, string nrcontribuinte);
			
		/// <summary>
		/// Requerer todas as chamadas
		/// </summary>
		/// <param name="user"> nome do utilizador valido </param>
		/// <param name="pwd"> password do utilizador</param>
		/// <post> nao alterao dos dados </post>
		/// <returns>retorna todas as chamadas</returns>
		DataSet GetTodas(string user, string pwd);
		
		/// <summary>
        /// Registar o inicio de uma chamada
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="inicio"> data de inicio da chamada </param>
        /// <param name="nrtelefone"> numero do telefone que se pretende o registo das chamadas, valido</param>
        /// <param name="numdestino"> numero de destino da chamada </param>
        /// <param name="id"> identificador da chamada </param>
        /// <pre> utilizador valido, numero de telefone valido, data inicio valida </pre>
        /// <post> identificador da chamada valido </post>
        /// <returns> retorna o estado da chamada e o identificador da chamada</returns>
        StatusEnum Inicio(string user, string pwd, DateTime inicio, string nrtelefone, string numdestino, out long id);

        /// <summary>
        /// finaliza uma chamada em curso
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="fim"> data em que finaliza a chamada </param>
        /// <param name="id"> identificador da chamada a finalizar valido </param>
        /// <pre> utilizador valido, data fim valida, identificador da chamada valido </pre>
        /// <post> chamada finalizada </post>
        /// <returns> estado da operao </returns>
        StatusEnum Fim(string user, string pwd, DateTime fim, long id);
    }

    internal class ChamadaImpl : IChamada
    {
        public ChamadaImpl()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Permite inserir uma chamada num determinado carto
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador</param>
        /// <param name="inicio"> data e hora do inicio da chamada</param>
        /// <param name="fim"> data e hora do final da chamada</param>
        /// <param name="nrtelefone"> numero do telefone que efectua a chamada valido </param>
        /// <param name="numdestino"> numero de destino da chamada </param>
        /// <pre> utilizador valido, numero de telefone valido, data inicio e data fim validas</pre>
        /// <post> chamada inserida </post>
        /// <returns> retorna o estado da insero da chamada </returns>
        public StatusEnum Chamada(string user, string pwd, DateTime inicio, DateTime fim, string nrtelefone, string numdestino,out long id)
        {
			StatusEnum status = StatusEnum.NOT_OK;
			OleDbConnection conn = null;
			OleDbTransaction  tx = null;
			TipoUtilizador tipoutil;
            
			id = -1;

			//
			// validar dados de entrada
			//
			if (fim <= inicio)
			{
				return status = StatusEnum.DATAHORA_INVALIDA;
			}
			if (nrtelefone == null || nrtelefone.Trim().Length == 0)
			{
				return status = StatusEnum.INVALID_KEY;
			}
			if (numdestino == null || numdestino.Trim().Length == 0)
			{
				return status = StatusEnum.NRTELEFONE_INVALIDO;
			}

			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// iniciar transaco
				//
				tx = conn.BeginTransaction();

				//
				// validar o utilizador
				//
				//status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				status = UtilDB.ValidateUser(conn, tx, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//verificar se este nrTelefone esta atribudo e se est ACTIVO
				DataSet ds = UtilDB.GetById(conn, tx, "Cartao", "nrTelefone", nrtelefone);
				if (ds.Tables["Cartao"].Rows.Count == 0)
				{
					return StatusEnum.INVALID_KEY;
				}
				else
				{
					Int16 estado = (Int16)ds.Tables["Cartao"].Rows[0]["estado"];
					if((EstadosCartao)estado != EstadosCartao.ACTIVO)
					{
						// Carto no est ACTIVO logo no pode efectuar chamadas
						return StatusEnum.ERRO_ESTADO_CARTAO;
					}
				}

				// ======================================
				//
				// tratar das inseres
				//
				// ======================================

				//
				// Determinar o custo da chamada
				//
				double custo = CustoDaChamada(inicio,fim,nrtelefone,numdestino);
				
				//
				// criar comando para insero
				//
				OleDbCommand cmd = new OleDbCommand("INSERT INTO Chamada (nrTelefone,nrTelDestino,inicio,fim,custo,terminada) VALUES (?,?,?,?,?,?)", conn);
				cmd.Transaction = tx;
				cmd.Parameters.Add("nrTelefone", nrtelefone);
				cmd.Parameters.Add("nrTelDestino", numdestino);
				OleDbParameter parm = cmd.Parameters.Add("inicio", OleDbType.Date);
				parm.Value = inicio;
				parm = cmd.Parameters.Add("fim", OleDbType.Date);
				parm.Value = fim;
				cmd.Parameters.Add("custo", custo);
				cmd.Parameters.Add("terminada", true);

				//
				// inserir registo: tarifario
				//
				cmd.ExecuteNonQuery();

				//
				// obter id autogerado ( que tambm ser retornado! )
				//
				OleDbCommand cmd2 = new OleDbCommand("SELECT @@IDENTITY", conn);
				cmd2.Transaction = tx;
				id = (int)cmd2.ExecuteScalar();

				//
				// fechar transaco
				//
				tx.Commit();
			}
			catch(OleDbException ex)
			{
				//
				// descartar mudanas efectuadas
				//
				if (tx != null)
					tx.Rollback();

				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Criar> da classe 'Tarifario': " + ex.Message);
			}
			finally
			{
				conn.Close();
			}

			/**/

			return status = StatusEnum.OK;
		}

        /// <summary>
        /// Requerer os dados das chamadas de um carto
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador</param>
        /// <param name="nrtelefone"> numero do telefone que se pretende o registo das chamadas, valido</param>
        /// <pre> utilizador valido, numero de telefone valido </pre>
        /// <post> nao alterao dos dados </post>
        /// <returns>retorna os dados das chamadas efectuadas por um determinado numero de telefone </returns>
        public DataSet GetByCartao(string user, string pwd, string nrtelefone)
        {
			OleDbConnection conn = null;
			DataSet rs = null;
			TipoUtilizador tipoutil;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar utilizar
				//
				if( UtilDB.ValidateUser(conn,null,user,pwd,out tipoutil) != StatusEnum.OK )
					return null;

				//
				// efectuar pesquisa
				//
				rs = UtilDB.GetById(conn, null, "Chamada", "nrTelefone", nrtelefone);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetByCartao> da classe 'Chamada': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return rs;
		}

		/// <summary>
		/// Requerer os dados das chamadas de um carto
		/// </summary>
		/// <param name="user"> nome do utilizador valido </param>
		/// <param name="pwd"> password do utilizador</param>
		/// <param name="nrcontribuinte"> numero de contribuinte que se pretende o registo das chamadas, valido</param>
		/// <post> nao alterao dos dados </post>
		/// <returns>retorna os dados das chamadas efectuadas por um determinado numero de telefone </returns>
		public DataSet GetByCliente(string user, string pwd, string nrcontribuinte)
		{
			OleDbConnection conn = null;
			DataSet rs = null;
			TipoUtilizador tipoutil;

			try
			{
				//
				// criar objecto DataSet
				//
				rs = new DataSet();

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar utilizar
				//
				if( UtilDB.ValidateUser(conn,null,user,pwd,out tipoutil) != StatusEnum.OK )
					return null;

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// linhas de detalhe (Tarifa)
				//
				string sSqlCmd = "SELECT * FROM Chamada WHERE nrTelefone IN (SELECT nrTelefone FROM Cartao WHERE nrContrib=?)";
				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Parameters.Add("nrContrib", nrcontribuinte);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(rs,"Chamada");
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetByCliente> da classe 'Chamada': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return rs;
		}

		/// <summary>
		/// Requerer todas as chamadas
		/// </summary>
		/// <param name="user"> nome do utilizador valido </param>
		/// <param name="pwd"> password do utilizador</param>
		/// <post> nao alterao dos dados </post>
		/// <returns>retorna todas as chamadas</returns>
		public DataSet GetTodas(string user, string pwd)
		{
			OleDbConnection conn = null;
			DataSet rs = null;
			TipoUtilizador tipoutil;

			try
			{
				//
				// criar objecto DataSet
				//
				rs = new DataSet();

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar utilizar
				//
				if( UtilDB.ValidateUser(conn,null,user,pwd,out tipoutil) != StatusEnum.OK )
					return null;

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// linhas de detalhe (Tarifa)
				//
				string sSqlCmd = "SELECT * FROM Chamada";
				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(rs,"Chamada");
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetTodas> da classe 'Chamada': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return rs;
		}

		/// <summary>
        /// Registar o inicio de uma chamada
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="inicio"> data de inicio da chamada </param>
        /// <param name="nrtelefone"> numero do telefone que se pretende o registo das chamadas, valido</param>
        /// <param name="numdestino"> numero de destino da chamada </param>
        /// <param name="id"> identificador da chamada </param>
        /// <pre> utilizador valido, numero de telefone valido, data inicio valida </pre>
        /// <post> identificador da chamada valido </post>
        /// <returns> retorna o estado da chamada e o identificador da chamada</returns>
        public StatusEnum Inicio(string user, string pwd, DateTime inicio, string nrtelefone, string numdestino, out long id)
        {
			StatusEnum status = StatusEnum.NOT_OK;
			OleDbConnection conn = null;
			OleDbTransaction  tx = null;
			TipoUtilizador tipoutil;
            
			id = -1;

			//
			// validar dados de entrada
			//
			if (nrtelefone == null || nrtelefone.Trim().Length == 0)
			{
				return status = StatusEnum.INVALID_KEY;
			}
			if (numdestino == null || numdestino.Trim().Length == 0)
			{
				return status = StatusEnum.NRTELEFONE_INVALIDO;
			}

			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// iniciar transaco
				//
				tx = conn.BeginTransaction();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, tx, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//verificar se este nrTelefone esta atribudo e se est ACTIVO
				DataSet ds = UtilDB.GetById(conn, tx, "Cartao", "nrTelefone", nrtelefone);
				if (ds.Tables["Cartao"].Rows.Count == 0)
				{
					return StatusEnum.INVALID_KEY;
				}
				else
				{
					Int16 estado = (Int16)ds.Tables["Cartao"].Rows[0]["estado"];
					if((EstadosCartao)estado != EstadosCartao.ACTIVO)
					{
						// Carto no est ACTIVO logo no pode efectuar chamadas
						return StatusEnum.ERRO_ESTADO_CARTAO;
					}
				}

				// Um nrtelefone no pode efectuar mais do que uma chamada em simultneo
				OleDbCommand cmd = new OleDbCommand("SELECT * FROM Chamada WHERE nrTelefone=? AND terminada=false", conn);
				cmd.Transaction = tx;
				cmd.Parameters.Add("nrTelefone",nrtelefone);
				int nRows = cmd.ExecuteNonQuery(); //executar comando
				if (nRows != 0)
				{
					//Est a efectuar uma chamada; retorna o erro
					return StatusEnum.NRTELEFONE_EMCHAMADA;
				}

				// ======================================
				//
				// tratar das inseres
				//
				// ======================================

				double custo = 0.0;

				//
				// criar comando para insero
				//
				OleDbCommand cmd2 = new OleDbCommand("INSERT INTO chamada (nrTelefone,nrTelDestino,inicio,fim,custo,terminada) VALUES (?,?,?,?,?,?)", conn);
				cmd2.Transaction = tx;
				cmd2.Parameters.Add("nrTelefone", nrtelefone);
				cmd2.Parameters.Add("nrTelDestino", numdestino);
				OleDbParameter parm = cmd2.Parameters.Add("inicio", OleDbType.Date);
				parm.Value = inicio;
				parm = cmd2.Parameters.Add("fim", OleDbType.Date);
				parm.Value = inicio; //para j fica = ao inicio
				cmd2.Parameters.Add("custo", custo);
				cmd2.Parameters.Add("terminada", false);

				//
				// inserir registo: tarifario
				//
				cmd2.ExecuteNonQuery();

				//
				// obter id autogerado ( que tambm ser retornado! )
				//
				OleDbCommand cmd3 = new OleDbCommand("SELECT @@IDENTITY", conn);
				cmd3.Transaction = tx;
				id = (int)cmd3.ExecuteScalar();

				//
				// fechar transaco
				//
				tx.Commit();
			}
			catch(OleDbException ex)
			{
				//
				// descartar mudanas efectuadas
				//
				if (tx != null)
					tx.Rollback();

				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Inicio> da classe 'Chamada': " + ex.Message);
			}
			finally
			{
				conn.Close();
			}

			/**/

			return status = StatusEnum.OK;
		}

        /// <summary>
        /// finaliza uma chamada em curso
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="fim"> data em que finaliza a chamada </param>
        /// <param name="id"> identificador da chamada a finalizar valido </param>
        /// <pre> utilizador valido, data fim valida, identificador da chamada valido </pre>
        /// <post> chamada finalizada </post>
        /// <returns> estado da operao </returns>
        public StatusEnum Fim(string user, string pwd, DateTime fim, long id)
        {
			StatusEnum status = StatusEnum.NOT_OK;
			OleDbConnection conn = null;
			OleDbTransaction  tx = null;
			TipoUtilizador tipoutil;
            
			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// iniciar transaco
				//
				tx = conn.BeginTransaction();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, tx, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//verificar se a Chamada existe e se no est j terminada
				DataSet ds = UtilDB.GetById(conn, tx, "Chamada", "id", id);
				if (ds.Tables["Chamada"].Rows.Count == 0)
				{
					return StatusEnum.INVALID_KEY;
				}
				else
				{
					if ((bool)ds.Tables["Chamada"].Rows[0]["terminada"] == true)
					{
						// Chamada j foi terminada!
						return StatusEnum.INVALID_KEY;
					}
				}

				DateTime inicio = Convert.ToDateTime(ds.Tables["Chamada"].Rows[0]["inicio"]);
				if (fim < inicio)
				{
					return StatusEnum.NOT_OK;
				}

				// ======================================
				//
				// tratar da actualizao
				//
				// ======================================

				//
				// Determinar o custo da chamada
				//
				string nrtelefone = ds.Tables["Chamada"].Rows[0]["nrTelefone"].ToString();
				string numdestino = ds.Tables["Chamada"].Rows[0]["nrTelDestino"].ToString();
				double custo = CustoDaChamada(inicio,fim,nrtelefone,numdestino);

				// Um nrtelefone no pode efectuar mais do que uma chamada em simultneo
				OleDbCommand cmd = new OleDbCommand("UPDATE Chamada SET fim=?, custo=?, terminada=? WHERE id=?", conn);
				cmd.Transaction = tx;
				OleDbParameter parm = cmd.Parameters.Add("fim", OleDbType.Date);
				parm.Value = fim;
				cmd.Parameters.Add("custo",custo);
				cmd.Parameters.Add("terminada",true);
				cmd.Parameters.Add("id",id);
				int nRows = cmd.ExecuteNonQuery(); //executar comando
				if (nRows == 0)
				{
					//Est a efectuar uma chamada; retorna o erro
					return status = StatusEnum.NOT_OK;
				}

				//
				// fechar transaco
				//
				tx.Commit();
			}
			catch(OleDbException ex)
			{
				//
				// descartar mudanas efectuadas
				//
				if (tx != null)
					tx.Rollback();

				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Fim> da classe 'Chamada': " + ex.Message);
			}
			finally
			{
				conn.Close();
			}

			/**/

			return status = StatusEnum.OK;
		}

		internal bool DentroDoPeriodo(DateTime tm, DiaDaSemana iniDia, DiaDaSemana fimDia, DateTime iniHr, DateTime fimHr)
		{
			//
			// Vou ter de contemplar os feriados
			//

			bool bRet = false;
			DayOfWeek ini = (DayOfWeek) iniDia;
			DayOfWeek fim = (DayOfWeek) fimDia;

			if (((tm.DayOfWeek.CompareTo(ini)>=0) && (tm.DayOfWeek.CompareTo(fim)<=0)) ||
				((tm.DayOfWeek.CompareTo(fim)>=0) && (tm.DayOfWeek.CompareTo(ini)<=0))) 
			{
				if (((tm.TimeOfDay.CompareTo(iniHr.TimeOfDay)>=0)&&(tm.TimeOfDay.CompareTo(fimHr.TimeOfDay)<=0)) ||
					((tm.TimeOfDay.CompareTo(fimHr.TimeOfDay)>=0)&&(tm.TimeOfDay.CompareTo(iniHr.TimeOfDay)<=0))) 
				{
					bRet = true;
				}
			}

			return bRet;
		}

		/// <summary>
		/// Calcula o custo da chamada
		/// </summary>
		/// <param name="inicio"> data e hora do inicio da chamada</param>
		/// <param name="fim"> data e hora do final da chamada</param>
		/// <param name="nrtelefone"> numero do telefone que efectua a chamada valido </param>
		/// <param name="numdestino"> numero de destino da chamada </param>
		/// <pre> utilizador valido, numero de telefone valido, data inicio e data fim validas</pre>
		/// <post> chamada inserida </post>
		/// <returns> retorna o custo da chamada </returns>
		private double CustoDaChamada(DateTime inicio, DateTime fim, string nrtelefone, string numdestino)
		{
			double custo = 0.0;

			// a implementar
			OleDbConnection conn = null;
			DataSet dsTarifas = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// efectuar pesquisa
				//
				DataSet dsCartoes = UtilDB.GetById(conn, null, "Cartao", "nrTelefone", nrtelefone);
				if ((dsCartoes == null) || (dsCartoes.Tables["Cartao"].Rows.Count == 0))
				{
					return custo;
				}

				//
				// efectuar pesquisa
				//
				long idTarifario = (int)(dsCartoes.Tables["Cartao"].Rows[0]["tarifario"]);
				dsTarifas = UtilDB.GetById(conn, null, "Tarifa", "tarifario",idTarifario);
				if ((dsTarifas == null) || (dsTarifas.Tables["Tarifa"].Rows.Count == 0))
				{
					return custo;
				}

				DateTime fimCursor, iniCursor = inicio;
				while (iniCursor < fim)
				{
					fimCursor = iniCursor;
					foreach (DataRow regTarifa in dsTarifas.Tables["Tarifa"].Rows)
					{
						DiaDaSemana iniDia= (DiaDaSemana) ((byte) regTarifa["diaSemanaIni"]);
						DiaDaSemana fimDia= (DiaDaSemana) ((byte) regTarifa["diaSemanaFim"]);
						DateTime hrInicio = Convert.ToDateTime(regTarifa["horaInicio"]);
						DateTime hrFim    = Convert.ToDateTime(regTarifa["horaFim"]);
						TimeSpan tmDiff;
						double nrPeriodos,segundos;

						if (DentroDoPeriodo(iniCursor,iniDia,fimDia,hrInicio,hrFim))
						{
							if (DentroDoPeriodo(fim,iniDia,fimDia,hrInicio,hrFim))
							{
								tmDiff = fim.TimeOfDay - iniCursor.TimeOfDay;
							}
							else
							{
								tmDiff = hrFim.TimeOfDay - iniCursor.TimeOfDay;
							}
							//
							// Vamos contar perdos inteiros de segundos. Por exemplo se a taxao for
							// a cada 30 segundos => 45 segundos = 1,5 perodos => 2 perodos
							// (menor nmero inteiro no menor que 1,5)
							//
							
							segundos = (short)regTarifa["segundos"];
							nrPeriodos = Math.Ceiling(tmDiff.TotalSeconds / segundos);
							custo += (nrPeriodos*(double)regTarifa["custo"]);
							
							/*
							fimCursor = iniCursor;
							fimCursor.Add(System.TimeSpan.FromSeconds(nrPeriodos*segundos));
							*/
							/*
							fimCursor = iniCursor;
							TimeSpan tmSpan = System.TimeSpan.FromSeconds(nrPeriodos*segundos);
							fimCursor.Add(tmSpan);
							*/
							
							fimCursor = iniCursor.AddSeconds(nrPeriodos*segundos);
							
							break;
						}
					}
					iniCursor = fimCursor;
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <CustoDaChamada> da classe 'Tarifario': " + ex.Message);
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return custo;
		}
    }
}
