using System;
using System.Collections;
using System.Text;
using System.Data;
using System.Data.OleDb;

namespace Telecomunicacao
{
    public interface ICliente
    {
        /// <summary>
        /// Criar um cliente
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador valido </param>
        /// <param name="contribuinte"> numero de contribuinte do cliente </param>
        /// <param name="bi"> numero do bilhete de identidade do cliente vlido</param>
        /// <param name="nome"> nome do cliente </param>
        /// <param name="morada"> morada do cliente </param>
        /// <pre> utilizador valido, e numero de bilhete de identidade valido </pre>
        /// <post> novo cliente inserido </post>
        /// <returns> retorna o estado da operao de insero do tipo StatusEnum</returns>
        StatusEnum criar(string user, string pwd, string contribuinte, string bi, string nome, string morada);

        /// <summary>
        /// permite alterar os dados de um cliente
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="contribuinte"> numero de contribuinte do cliente </param>
        /// <param name="bi"> numero do bilhete de identidade do cliente vlido</param>
        /// <param name="nome"> nome do cliente </param>
        /// <param name="morada"> morada do cliente </param>
        /// <pre> utilizador valido e o numero do bilhete de identidade do cliente valido </pre>
        /// <post> dados do cliente alterados </post>
        /// <returns> retorna o estado da operao de alterao</returns>
        StatusEnum alterar(string user, string pwd, string contribuinte, string bi, string nome, string morada);

        /// <summary>
        /// remover um cliente
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="bi"> numero de contribuinte (vlido) do cliente a remover</param>
        /// <pre> utilizador valido, numero de bilhete de identidade do cliente valido,
        /// o cliente no pode ter debitos </pre>
        /// <post> o cliente  removido </post>
        /// <returns>retorna o estado da operao de remoo </returns>
        StatusEnum remover(string user, string pwd, string contribuinte);
    }

    internal class ClienteImpl : ICliente
    {
		#region ICliente

        /// <summary>
        /// Criar um cliente
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador valido </param>
        /// <param name="contribuinte">contribuinte do cliente vlido</param>
        /// <param name="bi"> numero do bilhete de identidade do cliente vlido</param>
        /// <param name="nome"> nome do cliente </param>
        /// <param name="morada"> morada do cliente </param>
        /// <pre> utilizador valido, e numero de bilhete de identidade valido </pre>
        /// <post> novo cliente inserido </post>
        /// <returns> retorna o estado da operao de insero do tipo StatusEnum</returns>
        public StatusEnum criar(string user, string pwd, string contribuinte, string bi, string nome, string morada)
        {
            StatusEnum status;
			OleDbConnection conn = null;
            TipoUtilizador tipoutil;

			//
			// validar dados de entrada
			//
			//Carlos 13-1-2007
			if (nome == null || nome.Trim().Length == 0 || morada == null || morada.Trim().Length == 0)
			{
				status = StatusEnum.INVALID_ARGUMENT;
				return status;
			}

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				if (status!= StatusEnum.OK)
					return status;

				//
				// criar comando SQL a executar
				//
				string sqlCmd = "INSERT INTO Cliente (contribuinte, bi, nome, morada, activo) Values (?, ?, ?, ?, ?)";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);
				cmd.Parameters.Add("contribuinte", contribuinte);
				if (bi == null || bi.Length == 0)
					cmd.Parameters.Add("bi", DBNull.Value);
				else
					cmd.Parameters.Add("bi", bi);
				if (nome == null || nome.Length == 0)
					cmd.Parameters.Add("nome", DBNull.Value);
				else
					cmd.Parameters.Add("nome", nome);
				if (morada == null || morada.Length == 0)
					cmd.Parameters.Add("morada", DBNull.Value);
				else
					cmd.Parameters.Add("morada", morada);
				//Carlos 13-1-07
				cmd.Parameters.Add("activo",true);
				//
				// executar o comando 
				//
				int insRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = StatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					//
					status = StatusEnum.OK;		
				}
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <criar> da classe 'Cliente': " + ex.Message);
				status = StatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn != null && conn.State == ConnectionState.Open)
					conn.Close();
			}

            return status;
        }

        /// <summary>
        /// permite alterar os dados de um cliente
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="contribuinte">contribuinte do cliente</param>
        /// <param name="bi"> numero do bilhete de identidade do cliente vlido</param>
        /// <param name="nome"> nome do cliente </param>
        /// <param name="morada"> morada do cliente </param>
        /// <pre> utilizador valido e o numero de contribuinte do cliente valido </pre>
        /// <post> dados do cliente alterados </post>
        /// <returns> retorna o estado da operao de alterao</returns>
        public StatusEnum alterar(string user, string pwd, string contribuinte, string bi, string nome, string morada)
        {
			OleDbConnection conn = null;
			StatusEnum status;
            TipoUtilizador tipoutil;

			//
			// validar dados de entrada
			//
			//Carlos 13-1-2007
			if (nome == null || nome.Trim().Length == 0 || morada == null || morada.Trim().Length == 0)
			{
				status = StatusEnum.INVALID_ARGUMENT;
				return status;
			}

			//Carlos 13-1-2007
			//verificar se o cliente existe
			DataSet ds = UtilDB.GetById(conn, null, "Cliente", "contribuinte", contribuinte);
			if (ds.Tables["Cliente"].Rows.Count == 0)
				return StatusEnum.INVALID_CLIENTE_ID;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
				if (status != StatusEnum.OK)
					return status;

				//
				// criar comando SQL a executar
				//
				string sSqlCmd = "Update Cliente SET nome = ?, morada = ?, bi = ? WHERE contribuinte = ?";
				OleDbCommand cmd = new OleDbCommand(sSqlCmd, conn);
				if (nome.Length == 0)
					cmd.Parameters.Add("nome", DBNull.Value);
				else
					cmd.Parameters.Add("nome", nome);
				if (morada.Length == 0)
					cmd.Parameters.Add("morada", DBNull.Value);
				else
					cmd.Parameters.Add("morada", morada);
				if (bi.Length == 0)
					cmd.Parameters.Add("bi", DBNull.Value);
				else
					cmd.Parameters.Add("bi", bi);
				cmd.Parameters.Add("contribuinte", contribuinte);

				//
				// executar o comando 
				//
				int updRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (updRows == 0)
					// no actualizou registo
					status = StatusEnum.NOT_OK;
				else
					// actualizou registo
					status = StatusEnum.OK;
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Alterar> da classe 'Cliente': " + ex.Message);
				status = StatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
        }

        /// <summary>
        /// remover um cliente
        /// </summary>
        /// <param name="user"> nome do utilizador valido </param>
        /// <param name="pwd"> password do utilizador </param>
        /// <param name="bi"> numero de contribuinte (vlido) do cliente a remover</param>
        /// <pre> utilizador valido, numero de bilhete de identidade do cliente valido,
        /// o cliente no pode ter debitos </pre>
        /// <post> o cliente  removido </post>
        /// <returns>retorna o estado da operao de remoo </returns>
        public StatusEnum remover(string user, string pwd, string contribuinte) 
        {
			OleDbConnection conn = null;
            StatusEnum status = StatusEnum.NOT_OK;
            TipoUtilizador tipoutil;
            double quantia;

            try
            {
                //
                // criar objecto de conexo  base de dados e abrir a conexo
                //
                conn = new OleDbConnection(UtilDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                status = UtilDB.ValidateUser(conn, null, user, pwd, out tipoutil);
                if (status != StatusEnum.OK)
                    return status;

                //verificar se o cliente existe
                DataSet ds = UtilDB.GetById(conn, null, "Cliente", "contribuinte", contribuinte);
                if (ds.Tables["Cliente"].Rows.Count == 0)
                    return StatusEnum.INVALID_CLIENTE_ID;

                //verificar se o cliente tem debitos
                if (debitos(contribuinte, out quantia))
                    return StatusEnum.TEM_DEBITOS;

                //
                // criar comando SQL a executar
                //
                string sSqlCmd = "Update Cliente SET activo = ? WHERE contribuinte = ?";
                OleDbCommand cmd = new OleDbCommand(sSqlCmd, conn);
                cmd.Parameters.Add("activo", false);
                cmd.Parameters.Add("contribuinte", contribuinte);

                //
                // executar o comando 
                //
                int updRows = cmd.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (updRows == 0)
                    // no actualizou registo
                    status = StatusEnum.NOT_OK;
                else
                    // actualizou registo
                    status = StatusEnum.OK;
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepo!!!!
                //
                System.Console.WriteLine("EXCEPO no mtodo <Remover> da classe 'Cliente': " + ex.Message);
                status = StatusEnum.ERROR;
            }
            finally
            {
                //
                // fechar a conexo
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

			return status;
        }
		#endregion

		#region metodos internos
		/// <summary>
		/// permite saber se um cliente tem debitos
		/// </summary>
		/// <param name="contribuinte">contribuinte do cliente valido</param>
		/// <returns></returns>
		internal bool debitos(string contribuinte, out double quantia)
		{
            bool aux = true;
            quantia = 0;
            DataSet ds = GetCartoes(contribuinte);

            foreach (DataRow regCartoes in ds.Tables["Cartao"].Rows)
            {
                if (!(bool)regCartoes["recarregavel"])
                {
                    quantia += (double)regCartoes["saldo"];
                }
            }

            if (quantia == 0)
            {
                aux = false;
            }
            return aux;
		}

        internal DataSet GetCartoes(string contribuinte)
        {
            DataSet ds = null;
            OleDbConnection conn = null;

            try
            {
                //
                // criar objecto DataSet
                //
                ds = new DataSet();

                //
                // criar objecto de conexo  base de dados e abrir a conexo
                //
                conn = new OleDbConnection(UtilDB.CONN);
                conn.Open();

                //
                // criar um Datadapter para executar o comando e devolver o dataset
                //
                string sSqlCmd = "select * from Cartao where nrContrib='" + contribuinte + "'";
                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);

                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(ds, "Cartao");
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepo!!!!
                //
                System.Console.WriteLine("EXCEPO no mtodo 'GetCartoes': " + ex.Message);
                ds = null;
            }
            finally
            {
                //
                // fechar a conexo
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }
		#endregion
    }
}
